import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.StringTokenizer;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JTextField;

/**
 * Graphics - A graphics coursework in Java.
 *
 * I declare this work is mine in its entirety with the exception
 * of template code provided by Dr. Mark Jones of Swansea University.
 * 
 * @author Sean Handley (320097@swan.ac.uk)
 * @version 2006-02
 * 
 */
public class FilterDiag extends JDialog {
	private static final long serialVersionUID = 1L;
	JButton ok_button, cancel_button, preset_button, create_button, load_button;
	JTextField path_text;
	JLabel path_label;
	int filter[][];

	public FilterDiag() {
		super();
	}
	public int[][] makeDiag() {
		
		//GUI CODE BEGINS
		
		Container contentPane = getContentPane();
		contentPane.setLayout(new GridLayout(3,1));
		
		Container bottom_container = new Container();
		bottom_container.setLayout(new FlowLayout());
		
		Container mid_container = new Container();
		mid_container.setLayout(new BorderLayout());
		
		Container top_container = new Container();
		top_container.setLayout(new GridLayout(3,1));
		
		ok_button = new JButton("OK");
		bottom_container.add(ok_button);
		
		cancel_button = new JButton("Cancel");
		bottom_container.add(cancel_button);
		
		preset_button = new JButton("Presets");
		mid_container.add(preset_button,BorderLayout.NORTH);
		
		create_button = new JButton("Create");
		mid_container.add(create_button,BorderLayout.SOUTH);
		
		path_label = new JLabel("Path: ");
		top_container.add(path_label);
		
		path_text = new JTextField("path/to/filter.fil");
		top_container.add(path_text);
		
		load_button = new JButton("Browse");
		top_container.add(load_button);
		
		contentPane.add(top_container);
		contentPane.add(mid_container);
		contentPane.add(bottom_container);
		
		//GUI CODE ENDS
		
		GUIEventHandler handler = new GUIEventHandler();
		
		ok_button.addActionListener(handler);
		cancel_button.addActionListener(handler);
		load_button.addActionListener(handler);
		create_button.addActionListener(handler);
		
		pack();
		setModal(true);
		setTitle("Select Filter");
		setSize(450, 200);
		setLocationRelativeTo(null);
		setResizable(false);
		setVisible(true);
		
		//System.out.println("About to return filter...");
		return filter;
	}
	public void printFilter() {
		for(int i= 0;i<filter.length;i++) {
			for(int j=0; j< filter[i].length; j++) {
				System.out.print(filter[i][j] + "#");
			}
			System.out.println();
		}
	}
	public int[] getDimensions(String filename) {
		String thisLine;
		int inrow = 0, row = 0, col = 0;
		int dims[] = new int[2];
		
        try {
			BufferedReader reader = new BufferedReader(new FileReader(filename));
			while((thisLine = reader.readLine()) != null) {
				for(int i=0; i < thisLine.length(); i++){
					//System.out.println("char: " + thisLine.charAt(i));
					if(thisLine.charAt(i) == ':') {
						inrow++;
					}
				}
				col++;
				row = inrow;
				inrow = 0;
			}
			row++; // add one (a:b) = 1 colon + 1
			//System.out.println("row: " + row + ", col: " + col);
			//printFilter(); //use for testing
		} catch (FileNotFoundException e) {
			// do something
		} catch (IOException e) {
			// do something
		}
		dims[0] = row;
		dims[1] = col;
		
		return dims;
	}
	public int[][] loadFilter(String filename) {
		String thisLine;
		int i = 0, j = 0;
		int dims[] = getDimensions(filename);
		int x = dims[0];
		int y = dims[1];
		filter = new int[y][x];
        try {
			BufferedReader reader = new BufferedReader(new FileReader(filename));
			while((thisLine = reader.readLine()) != null) {
				StringTokenizer st = new StringTokenizer(thisLine,":");
				while(st.hasMoreTokens()){
					  String s = st.nextToken();
					  filter[i][j] = Integer.parseInt(s);
					  j++;
				}
				i++;
				j = 0;
			}
			//printFilter(); //use for testing
		} catch (FileNotFoundException e) {
			// do something
		} catch (IOException e) {
			// do something
		}
		return filter;
	}
	public class GUIEventHandler implements ActionListener {
		public void actionPerformed(ActionEvent event) {
			if(event.getSource() == ok_button) {
				loadFilter(path_text.getText());
				//kill the dialog
				dispose();
			} else if(event.getSource() == cancel_button) {
				//kill the dialog
				dispose();
			} else if(event.getSource() == load_button) {
				//load the filter
				String filename = "filter.fil";
				JFileChooser fc = new JFileChooser(new File(filename));
				
				// Show open dialog; this method does not return until the dialog is closed
				fc.showOpenDialog(rootPane);
				try {
					String selFile = fc.getSelectedFile().toString();
					path_text.setText(selFile);
				}
				catch(NullPointerException e) {
					//do nothing
				}
			} else if(event.getSource() == create_button) {
				DimsDiag dd = new DimsDiag("Enter dimensions for filter");
				dd.setModal(true);
				setPath(dd.make());
			}
		}
	}
	private void setPath(String path) {
		path_text.setText(path);
	}
}
